from loga import Loga

# all setup values are optional
loga = Loga(
    facility="info_ratio",  # name of program logging the message
    do_print=True,  # print each log to console
    truncation=1000,  # longest possible value in extra data
    private_data={"password"},  # set of sensitive args/kwargs
)


# @loga.errors
# def calculate_information_ratio(input_df, fund_info):

#     num_wins = 0
#     num_losses = 0
#     absolute_gain = 0
#     absolute_loss = 0

#     # Get the breadth of the fund
#     stock_list = input_df['Company Name'].unique()
#     breadth = len(stock_list)
#     if breadth == 0:
#         return fund_info, 4
#     fund_info.append(breadth)

#     for i in stock_list:
#         stock_df = input_df[input_df['Company Name'] == i]
#         stock_df["diff_tmp"] = stock_df["No Of Shares"].diff()
#         stock_df["ret_tmp"] = stock_df["Price"] * stock_df["diff_tmp"]
#         stock_df['ret_tmp'].iloc[0] = stock_df['Price'].iloc[0] * \
#             stock_df['No Of Shares'].iloc[0]

#         # We are assuming the the portfolio manager has exited the remaining shares at the end of the analysis period
#         last_exit = stock_df['Price'].iloc[-1] * \
#             stock_df['No Of Shares'].iloc[-1]

#         sum_of_positive_nos = stock_df[stock_df['ret_tmp']
#                                        >= 0]['ret_tmp'].sum()

#         some_of_negative_nos = np.abs(stock_df[stock_df['ret_tmp'] <= 0]
#                                       ['ret_tmp']).sum() + last_exit

#         if sum_of_positive_nos < some_of_negative_nos:
#             num_wins += 1
#             absolute_gain += round((some_of_negative_nos -
#                                    sum_of_positive_nos), 3)
#         else:
#             num_losses += 1
#             absolute_loss += round((some_of_negative_nos -
#                                    sum_of_positive_nos), 3)

#     if num_wins == 0:
#         return fund_info, 0
#     else:
#         average_gain = abs(absolute_gain)/num_wins
#     if num_losses == 0:
#         return fund_info, 1
#     else:
#         average_loss = abs(absolute_loss)/num_losses

#     fund_info.append(num_wins)
#     fund_info.append(num_losses)

#     # Pain-to-Gain Ratio
#     pain_gain_ratio = abs(absolute_gain/absolute_loss)

#     # Calculate the batting average
#     batting_average = num_wins/breadth
#     fund_info.append(round(batting_average, 3))

#     # Calculate the slugging ratio
#     slugging_ratio = average_gain/average_loss

#     fund_info.append(round(slugging_ratio, 3))

#     information_coefficient = 1.6*(batting_average - (1/(1+slugging_ratio)))
#     fund_info.append(round(information_coefficient, 3))

#     information_ratio = information_coefficient * (np.sqrt(breadth))
#     fund_info.append(round(information_ratio, 3))
#     fund_info.append(round(pain_gain_ratio, 3))

#     return fund_info, 3



import os
from info_ratio.common.logging import loga
import pandas as pd
from datetime import datetime
from info_ratio.common.input_output import dfs_tabs
from info_ratio.common.helper_functions import preprocess_data, calculate_information_ratio


pd.options.mode.chained_assignment = None  # default='warn'


@loga
class InfoRatio:
    def __init__(self, input_df, db_path, start_date, end_date, insert_data=False, enable_logging=True):

        if not enable_logging:
            loga.stop()

        self.main_dict = []
        self.only_wins = []
        self.only_losses = []
        self.dont_exist = []

        self.insert_data = insert_data

        self.input_df = input_df

        self.start_date = start_date
        self.end_date = end_date

        self.db_path = db_path

    def run(self):
        processed_df = preprocess_data(self)

        for scheme_name in self.input_df["Scheme Name"]:
            fund_info = [scheme_name]
            scheme_df = processed_df[processed_df["Scheme Name"]
                                     == scheme_name]

            fund_info, type_ = calculate_information_ratio(
                scheme_df, fund_info)
            if type_ == 0:
                self.only_losses.append(fund_info)
            elif type_ == 1:
                self.only_wins.append(fund_info)
            elif type_ == 4:
                self.dont_exist.append(fund_info)
            else:
                self.main_dict.append(fund_info)

        df = pd.DataFrame(self.main_dict, columns=['Fund Name', 'Breadth', 'Wins', 'Loss', "Batting Average",
                          "Slugging Ratio", "Information Coefficient", "Information Ratio", "Pain/Gain Ratio"])

        only_win_df = pd.DataFrame(self.only_wins, columns=[
            'Fund Name', 'Breadth'])
        only_loss_df = pd.DataFrame(self.only_losses, columns=[
                                    'Fund Name', 'Breadth'])
        dont_exist_df = pd.DataFrame(self.dont_exist, columns=['Fund Name'])

        return df, only_win_df, only_loss_df, dont_exist_df
